<?php

namespace Drupal\Tests\entity_access_by_role_field\Functional;

use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\entity_access_by_role_field\Helper\Constants;

/**
 * This class provides access tests for entity_access_by_role_field node.
 *
 * @group entity_access_by_role_field
 */
class NodeEntityAccessTests extends EntityAccessTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'entity_access_by_role_field',
    'node',
    'field',
  ];

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();

    // Create the adminUser:
    $this->createAdminRole('administrator', 'administrator');
    $this->adminUser = $this->createUser([
      'administer content types',
      'administer nodes',
    ]);
    $this->adminUser->addRole('administrator');

    // Create the global permissionUser:
    $this->globalPermissionUser = $this->createUser([
      'administer content types',
      'administer nodes',
      'entity_by_access_role_field.ignore_permission',
    ]);

    // Create the authenticatedUser:
    $this->authenticatedUser = $this->createUser([
      'administer content types',
      'administer nodes',
    ]);
    $this->authenticatedUser->addRole('authenticated_user');

    // Create the testRoleUser:
    $this->testRoleUser = $this->createUser([
      'administer content types',
    ]);
    $this->createRole(['administer nodes'], 'test_role', 'test_role');
    $this->testRoleUser->addRole('test_role');
  }

  /**
   * Test if global ignore_permission ignores field settings.
   */
  public function testGlobalPermissionOnForbiddenNode() {
    $this->drupalLogin($this->globalPermissionUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Check expected access BEFORE we add our field:
    $this->drupalGet('/node/' . $entity->id());
    $this->assertSession()->statusCodeEquals(200);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated_user'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, 'view', $this->globalPermissionUser);
  }

  /**
   * Test if accessing node as admin, ignores field settings.
   */
  public function testAdminAccessOnForbiddenNode() {
    $this->drupalLogin($this->adminUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['administrator'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, 'view', $this->adminUser);
  }

  /**
   * Check access on a node which allows authenticated users.
   */
  public function testAccessOnAllowedNodeAsAuth() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated_user'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, 'view', $this->authenticatedUser);
  }

  /**
   * Check access on a node which forbids authenticated users. TODO: Doesn't work
   */
  public function testAccessOnForbiddenNodeAsAuth() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode([
      'title' => 'Test Article',
      'type' => 'article',
      'field_access' => [
        'role_id' => 'authenticated_user',
        'access' => 'forbidden',
      ],
    ]);
    // Check expected access BEFORE we add our field:
    // $this->drupalGet('/node/' . $entity->id());
    // $this->assertSession()->statusCodeEquals(200);
    // Add our field:
    //$this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated_user'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, 'view', $this->authenticatedUser);
  }

  /**
   * Check access on a node which allows anonymous users, as a anonymous user.
   */
  public function testAccessOnAllowedNodeAsGuest() {
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['anonymous'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertTrue($entity->access('view'));
    $this->assertFalse($entity->access('view', $this->authenticatedUser));
    $this->assertAccessAllowed($entity);
    $this->assertAccessForbidden($entity, 'view', $this->authenticatedUser);
  }

  /**
   * Check access on a node which forbids anonymous users, as a anonymous user.
   */
  public function testAccessOnForbiddenNodeAsGuest() {
    // TODO: @JS: Use this as template for the other tests... works! Do not use the drupalGet for the access checks, something seems to be wrong in core...
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    // We create two entities here, as the access results otherwise
    // seem to be cached wrongly!
    $entity1 = $this->createNode([
      'title' => 'Test Article 1',
      'type' => 'article',
    ]);
    $this->assertAccessAllowed($entity1);

    // Add our field:
    $entity2 = $this->createNode([
      'title' => 'Test Article2',
      'type' => 'article',
    ]);
    $this->setEntityAccessByRoleFieldOnEntity($entity2, $this->fieldName, ['anonymous'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity2);
  }

  /**
   * Check access on a node which allows test_role users, as a test_role user.
   */
  public function testAccessOnAllowedNodeAsNewRole() {
    $this->drupalLogin($this->testRoleUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['test_role'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, 'view', $this->testRoleUser);
  }

  /**
   * Check access on a node which forbids test_role users, as a test_role user.
   */
  public function testAccessOnForbiddenNodeAsNewRole() {
    $this->drupalLogin($this->testRoleUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['test_role'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, 'view', $this->testRoleUser);
  }

  /**
   * Test to see if created roles are shown on creation of an article node.
   */
  public function testNodeFieldExists() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $session = $this->assertSession();
    $this->drupalGet('/node/add/article');
    $session->fieldExists("role_id[0]['test_role']");
    $session->fieldExists("role_id[0]['authenticated_user']");
    $session->fieldExists("role_id[0]['anonymous']");
    $session->fieldExists("role_id[0]['administrator']");

    // TODO: Ensure the radios exist
  }

  // @todo Test different operations and fllback values.
}
